#include "eb3516_nnie.h"

#include "hi_comm_svp.h"
#include "hi_comm_sys.h"
#include "hi_common.h"
#include "sample_comm.h"
#include "sample_comm_ive.h"
#include "sample_comm_nnie.h"
#include "sample_comm_svp.h"
#include "sample_svp_nnie_software.h"
#include "zlog.h"

static zlog_category_t *log_nnie = NULL;

/*yolov3 para*/
static SAMPLE_SVP_NNIE_MODEL_S g_stNnieModel = {0};
static SAMPLE_SVP_NNIE_PARAM_S g_stNnieParam = {0};
static SAMPLE_SVP_NNIE_YOLOV3_SOFTWARE_PARAM_S g_stYolov3Param = {0};

static HI_S32 SAMPLE_SVP_NNIE_Detection_PrintResult(
    SVP_BLOB_S *pstDstScore, SVP_BLOB_S *pstDstRoi, SVP_BLOB_S *pstClassRoiNum,
    HI_FLOAT f32PrintResultThresh) {
    HI_U32 i = 0, j = 0;
    HI_U32 u32RoiNumBias = 0;
    HI_U32 u32ScoreBias = 0;
    HI_U32 u32BboxBias = 0;
    HI_FLOAT f32Score = 0.0f;
    HI_S32 *ps32Score =
        SAMPLE_SVP_NNIE_CONVERT_64BIT_ADDR(HI_S32, pstDstScore->u64VirAddr);
    HI_S32 *ps32Roi =
        SAMPLE_SVP_NNIE_CONVERT_64BIT_ADDR(HI_S32, pstDstRoi->u64VirAddr);
    HI_S32 *ps32ClassRoiNum =
        SAMPLE_SVP_NNIE_CONVERT_64BIT_ADDR(HI_S32, pstClassRoiNum->u64VirAddr);
    HI_U32 u32ClassNum = pstClassRoiNum->unShape.stWhc.u32Width;
    HI_S32 s32XMin = 0, s32YMin = 0, s32XMax = 0, s32YMax = 0;

    u32RoiNumBias += ps32ClassRoiNum[0];
    for (i = 1; i < u32ClassNum; i++) {
        u32ScoreBias = u32RoiNumBias;
        u32BboxBias = u32RoiNumBias * SAMPLE_SVP_NNIE_COORDI_NUM;
        /*if the confidence score greater than result threshold, the result will
         * be printed*/
        if ((HI_FLOAT)ps32Score[u32ScoreBias] / SAMPLE_SVP_NNIE_QUANT_BASE >=
                f32PrintResultThresh &&
            ps32ClassRoiNum[i] != 0) {
            SAMPLE_SVP_TRACE_INFO("==== The %dth class box info====\n", i);
        }
        for (j = 0; j < (HI_U32)ps32ClassRoiNum[i]; j++) {
            f32Score = (HI_FLOAT)ps32Score[u32ScoreBias + j] /
                       SAMPLE_SVP_NNIE_QUANT_BASE;
            if (f32Score < f32PrintResultThresh) {
                break;
            }
            s32XMin = ps32Roi[u32BboxBias + j * SAMPLE_SVP_NNIE_COORDI_NUM];
            s32YMin = ps32Roi[u32BboxBias + j * SAMPLE_SVP_NNIE_COORDI_NUM + 1];
            s32XMax = ps32Roi[u32BboxBias + j * SAMPLE_SVP_NNIE_COORDI_NUM + 2];
            s32YMax = ps32Roi[u32BboxBias + j * SAMPLE_SVP_NNIE_COORDI_NUM + 3];
            SAMPLE_SVP_TRACE_INFO("%d %d %d %d %f\n", s32XMin, s32YMin, s32XMax,
                                  s32YMax, f32Score);
        }
        u32RoiNumBias += ps32ClassRoiNum[i];
    }
    return HI_SUCCESS;
}

static HI_S32 SAMPLE_SVP_NNIE_Yolov3_SoftwareDeinit(
    SAMPLE_SVP_NNIE_YOLOV3_SOFTWARE_PARAM_S *pstSoftWareParam) {
    HI_S32 s32Ret = HI_SUCCESS;
    SAMPLE_SVP_CHECK_EXPR_RET(NULL == pstSoftWareParam, HI_INVALID_VALUE,
                              SAMPLE_SVP_ERR_LEVEL_ERROR,
                              "Error, pstSoftWareParam can't be NULL!\n");
    if (0 != pstSoftWareParam->stGetResultTmpBuf.u64PhyAddr &&
        0 != pstSoftWareParam->stGetResultTmpBuf.u64VirAddr) {
        SAMPLE_SVP_MMZ_FREE(pstSoftWareParam->stGetResultTmpBuf.u64PhyAddr,
                            pstSoftWareParam->stGetResultTmpBuf.u64VirAddr);
        pstSoftWareParam->stGetResultTmpBuf.u64PhyAddr = 0;
        pstSoftWareParam->stGetResultTmpBuf.u64VirAddr = 0;
        pstSoftWareParam->stDstRoi.u64PhyAddr = 0;
        pstSoftWareParam->stDstRoi.u64VirAddr = 0;
        pstSoftWareParam->stDstScore.u64PhyAddr = 0;
        pstSoftWareParam->stDstScore.u64VirAddr = 0;
        pstSoftWareParam->stClassRoiNum.u64PhyAddr = 0;
        pstSoftWareParam->stClassRoiNum.u64VirAddr = 0;
    }
    return s32Ret;
}

static HI_S32 SAMPLE_SVP_NNIE_Yolov3_Deinit(
    SAMPLE_SVP_NNIE_PARAM_S *pstNnieParam,
    SAMPLE_SVP_NNIE_YOLOV3_SOFTWARE_PARAM_S *pstSoftWareParam,
    SAMPLE_SVP_NNIE_MODEL_S *pstNnieModel) {
    HI_S32 s32Ret = HI_SUCCESS;
    /*hardware deinit*/
    if (pstNnieParam != NULL) {
        s32Ret = SAMPLE_COMM_SVP_NNIE_ParamDeinit(pstNnieParam);
        SAMPLE_SVP_CHECK_EXPR_TRACE(
            HI_SUCCESS != s32Ret, SAMPLE_SVP_ERR_LEVEL_ERROR,
            "Error,SAMPLE_COMM_SVP_NNIE_ParamDeinit failed!\n");
    }
    /*software deinit*/
    if (pstSoftWareParam != NULL) {
        s32Ret = SAMPLE_SVP_NNIE_Yolov3_SoftwareDeinit(pstSoftWareParam);
        SAMPLE_SVP_CHECK_EXPR_TRACE(
            HI_SUCCESS != s32Ret, SAMPLE_SVP_ERR_LEVEL_ERROR,
            "Error,SAMPLE_SVP_NNIE_Yolov3_SoftwareDeinit failed!\n");
    }
    /*model deinit*/
    if (pstNnieModel != NULL) {
        s32Ret = SAMPLE_COMM_SVP_NNIE_UnloadModel(pstNnieModel);
        SAMPLE_SVP_CHECK_EXPR_TRACE(
            HI_SUCCESS != s32Ret, SAMPLE_SVP_ERR_LEVEL_ERROR,
            "Error,SAMPLE_COMM_SVP_NNIE_UnloadModel failed!\n");
    }
    return s32Ret;
}

static HI_S32 SAMPLE_SVP_NNIE_Yolov3_SoftwareInit(
    SAMPLE_SVP_NNIE_CFG_S *pstCfg, SAMPLE_SVP_NNIE_PARAM_S *pstNnieParam,
    SAMPLE_SVP_NNIE_YOLOV3_SOFTWARE_PARAM_S *pstSoftWareParam) {
    HI_S32 s32Ret = HI_SUCCESS;
    HI_U32 u32ClassNum = 0;
    HI_U32 u32TotalSize = 0;
    HI_U32 u32DstRoiSize = 0;
    HI_U32 u32DstScoreSize = 0;
    HI_U32 u32ClassRoiNumSize = 0;
    HI_U32 u32TmpBufTotalSize = 0;
    HI_U64 u64PhyAddr = 0;
    HI_U8 *pu8VirAddr = NULL;

    pstSoftWareParam->u32OriImHeight =
        pstNnieParam->astSegData[0].astSrc[0].unShape.stWhc.u32Height;
    pstSoftWareParam->u32OriImWidth =
        pstNnieParam->astSegData[0].astSrc[0].unShape.stWhc.u32Width;
    pstSoftWareParam->u32BboxNumEachGrid = 3;
    pstSoftWareParam->u32ClassNum = 80;
    pstSoftWareParam->au32GridNumHeight[0] = 13;
    pstSoftWareParam->au32GridNumHeight[1] = 26;
    pstSoftWareParam->au32GridNumHeight[2] = 52;
    pstSoftWareParam->au32GridNumWidth[0] = 13;
    pstSoftWareParam->au32GridNumWidth[1] = 26;
    pstSoftWareParam->au32GridNumWidth[2] = 52;
    pstSoftWareParam->u32NmsThresh =
        (HI_U32)(0.3f * SAMPLE_SVP_NNIE_QUANT_BASE);
    pstSoftWareParam->u32ConfThresh =
        (HI_U32)(0.5f * SAMPLE_SVP_NNIE_QUANT_BASE);
    pstSoftWareParam->u32MaxRoiNum = 10;
    pstSoftWareParam->af32Bias[0][0] = 116;
    pstSoftWareParam->af32Bias[0][1] = 90;
    pstSoftWareParam->af32Bias[0][2] = 156;
    pstSoftWareParam->af32Bias[0][3] = 198;
    pstSoftWareParam->af32Bias[0][4] = 373;
    pstSoftWareParam->af32Bias[0][5] = 326;
    pstSoftWareParam->af32Bias[1][0] = 30;
    pstSoftWareParam->af32Bias[1][1] = 61;
    pstSoftWareParam->af32Bias[1][2] = 62;
    pstSoftWareParam->af32Bias[1][3] = 45;
    pstSoftWareParam->af32Bias[1][4] = 59;
    pstSoftWareParam->af32Bias[1][5] = 119;
    pstSoftWareParam->af32Bias[2][0] = 10;
    pstSoftWareParam->af32Bias[2][1] = 13;
    pstSoftWareParam->af32Bias[2][2] = 16;
    pstSoftWareParam->af32Bias[2][3] = 30;
    pstSoftWareParam->af32Bias[2][4] = 33;
    pstSoftWareParam->af32Bias[2][5] = 23;

    /*Malloc assist buffer memory*/
    u32ClassNum = pstSoftWareParam->u32ClassNum + 1;

    SAMPLE_SVP_CHECK_EXPR_RET(
        SAMPLE_SVP_NNIE_YOLOV3_REPORT_BLOB_NUM !=
            pstNnieParam->pstModel->astSeg[0].u16DstNum,
        HI_FAILURE, SAMPLE_SVP_ERR_LEVEL_ERROR,
        "Error,pstNnieParam->pstModel->astSeg[0].u16DstNum(%d) should be %d!\n",
        pstNnieParam->pstModel->astSeg[0].u16DstNum,
        SAMPLE_SVP_NNIE_YOLOV3_REPORT_BLOB_NUM);
    u32TmpBufTotalSize =
        SAMPLE_SVP_NNIE_Yolov3_GetResultTmpBuf(pstNnieParam, pstSoftWareParam);
    u32DstRoiSize =
        SAMPLE_SVP_NNIE_ALIGN16(u32ClassNum * pstSoftWareParam->u32MaxRoiNum *
                                sizeof(HI_U32) * SAMPLE_SVP_NNIE_COORDI_NUM);
    u32DstScoreSize = SAMPLE_SVP_NNIE_ALIGN16(
        u32ClassNum * pstSoftWareParam->u32MaxRoiNum * sizeof(HI_U32));
    u32ClassRoiNumSize = SAMPLE_SVP_NNIE_ALIGN16(u32ClassNum * sizeof(HI_U32));
    u32TotalSize = u32TotalSize + u32DstRoiSize + u32DstScoreSize +
                   u32ClassRoiNumSize + u32TmpBufTotalSize;
    s32Ret = SAMPLE_COMM_SVP_MallocCached("SAMPLE_YOLOV3_INIT", NULL,
                                          (HI_U64 *)&u64PhyAddr,
                                          (void **)&pu8VirAddr, u32TotalSize);
    SAMPLE_SVP_CHECK_EXPR_RET(HI_SUCCESS != s32Ret, s32Ret,
                              SAMPLE_SVP_ERR_LEVEL_ERROR,
                              "Error,Malloc memory failed!\n");
    memset(pu8VirAddr, 0, u32TotalSize);
    SAMPLE_COMM_SVP_FlushCache(u64PhyAddr, (void *)pu8VirAddr, u32TotalSize);

    /*set each tmp buffer addr*/
    pstSoftWareParam->stGetResultTmpBuf.u64PhyAddr = u64PhyAddr;
    pstSoftWareParam->stGetResultTmpBuf.u64VirAddr =
        (HI_U64)((HI_UL)pu8VirAddr);

    /*set result blob*/
    pstSoftWareParam->stDstRoi.enType = SVP_BLOB_TYPE_S32;
    pstSoftWareParam->stDstRoi.u64PhyAddr = u64PhyAddr + u32TmpBufTotalSize;
    pstSoftWareParam->stDstRoi.u64VirAddr =
        (HI_U64)((HI_UL)pu8VirAddr + u32TmpBufTotalSize);
    pstSoftWareParam->stDstRoi.u32Stride =
        SAMPLE_SVP_NNIE_ALIGN16(u32ClassNum * pstSoftWareParam->u32MaxRoiNum *
                                sizeof(HI_U32) * SAMPLE_SVP_NNIE_COORDI_NUM);
    pstSoftWareParam->stDstRoi.u32Num = 1;
    pstSoftWareParam->stDstRoi.unShape.stWhc.u32Chn = 1;
    pstSoftWareParam->stDstRoi.unShape.stWhc.u32Height = 1;
    pstSoftWareParam->stDstRoi.unShape.stWhc.u32Width =
        u32ClassNum * pstSoftWareParam->u32MaxRoiNum *
        SAMPLE_SVP_NNIE_COORDI_NUM;

    pstSoftWareParam->stDstScore.enType = SVP_BLOB_TYPE_S32;
    pstSoftWareParam->stDstScore.u64PhyAddr =
        u64PhyAddr + u32TmpBufTotalSize + u32DstRoiSize;
    pstSoftWareParam->stDstScore.u64VirAddr =
        (HI_U64)((HI_UL)pu8VirAddr + u32TmpBufTotalSize + u32DstRoiSize);
    pstSoftWareParam->stDstScore.u32Stride = SAMPLE_SVP_NNIE_ALIGN16(
        u32ClassNum * pstSoftWareParam->u32MaxRoiNum * sizeof(HI_U32));
    pstSoftWareParam->stDstScore.u32Num = 1;
    pstSoftWareParam->stDstScore.unShape.stWhc.u32Chn = 1;
    pstSoftWareParam->stDstScore.unShape.stWhc.u32Height = 1;
    pstSoftWareParam->stDstScore.unShape.stWhc.u32Width =
        u32ClassNum * pstSoftWareParam->u32MaxRoiNum;

    pstSoftWareParam->stClassRoiNum.enType = SVP_BLOB_TYPE_S32;
    pstSoftWareParam->stClassRoiNum.u64PhyAddr =
        u64PhyAddr + u32TmpBufTotalSize + u32DstRoiSize + u32DstScoreSize;
    pstSoftWareParam->stClassRoiNum.u64VirAddr =
        (HI_U64)((HI_UL)pu8VirAddr + u32TmpBufTotalSize + u32DstRoiSize +
                 u32DstScoreSize);
    pstSoftWareParam->stClassRoiNum.u32Stride =
        SAMPLE_SVP_NNIE_ALIGN16(u32ClassNum * sizeof(HI_U32));
    pstSoftWareParam->stClassRoiNum.u32Num = 1;
    pstSoftWareParam->stClassRoiNum.unShape.stWhc.u32Chn = 1;
    pstSoftWareParam->stClassRoiNum.unShape.stWhc.u32Height = 1;
    pstSoftWareParam->stClassRoiNum.unShape.stWhc.u32Width = u32ClassNum;

    return s32Ret;
}

static HI_S32 SAMPLE_SVP_NNIE_Yolov3_ParamInit(
    SAMPLE_SVP_NNIE_CFG_S *pstCfg, SAMPLE_SVP_NNIE_PARAM_S *pstNnieParam,
    SAMPLE_SVP_NNIE_YOLOV3_SOFTWARE_PARAM_S *pstSoftWareParam) {
    HI_S32 s32Ret = HI_SUCCESS;
    /*init hardware para*/
    s32Ret = SAMPLE_COMM_SVP_NNIE_ParamInit(pstCfg, pstNnieParam);
    SAMPLE_SVP_CHECK_EXPR_GOTO(
        HI_SUCCESS != s32Ret, INIT_FAIL_0, SAMPLE_SVP_ERR_LEVEL_ERROR,
        "Error(%#x),SAMPLE_COMM_SVP_NNIE_ParamInit failed!\n", s32Ret);

    /*init software para*/
    s32Ret = SAMPLE_SVP_NNIE_Yolov3_SoftwareInit(pstCfg, pstNnieParam,
                                                 pstSoftWareParam);
    SAMPLE_SVP_CHECK_EXPR_GOTO(
        HI_SUCCESS != s32Ret, INIT_FAIL_0, SAMPLE_SVP_ERR_LEVEL_ERROR,
        "Error(%#x),SAMPLE_SVP_NNIE_Yolov3_SoftwareInit failed!\n", s32Ret);

    return s32Ret;
INIT_FAIL_0:
    s32Ret =
        SAMPLE_SVP_NNIE_Yolov3_Deinit(pstNnieParam, pstSoftWareParam, NULL);
    SAMPLE_SVP_CHECK_EXPR_RET(
        HI_SUCCESS != s32Ret, s32Ret, SAMPLE_SVP_ERR_LEVEL_ERROR,
        "Error(%#x),SAMPLE_SVP_NNIE_Yolov3_Deinit failed!\n", s32Ret);
    return HI_FAILURE;
}

static HI_S32 SAMPLE_SVP_NNIE_FillSrcData(
    SAMPLE_SVP_NNIE_CFG_S *pstNnieCfg, SAMPLE_SVP_NNIE_PARAM_S *pstNnieParam,
    SAMPLE_SVP_NNIE_INPUT_DATA_INDEX_S *pstInputDataIdx) {
    FILE *fp = NULL;
    HI_U32 i = 0, j = 0, n = 0;
    HI_U32 u32Height = 0, u32Width = 0, u32Chn = 0, u32Stride = 0, u32Dim = 0;
    HI_U32 u32VarSize = 0;
    HI_S32 s32Ret = HI_SUCCESS;
    HI_U8 *pu8PicAddr = NULL;
    HI_U32 *pu32StepAddr = NULL;
    HI_U32 u32SegIdx = pstInputDataIdx->u32SegIdx;
    HI_U32 u32NodeIdx = pstInputDataIdx->u32NodeIdx;
    HI_U32 u32TotalStepNum = 0;

    /*open file*/
    if (NULL != pstNnieCfg->pszPic) {
        fp = fopen(pstNnieCfg->pszPic, "rb");
        SAMPLE_SVP_CHECK_EXPR_RET(NULL == fp, HI_INVALID_VALUE,
                                  SAMPLE_SVP_ERR_LEVEL_ERROR,
                                  "Error, open file failed!\n");
    }

    /*get data size*/
    if (SVP_BLOB_TYPE_U8 <=
            pstNnieParam->astSegData[u32SegIdx].astSrc[u32NodeIdx].enType &&
        SVP_BLOB_TYPE_YVU422SP >=
            pstNnieParam->astSegData[u32SegIdx].astSrc[u32NodeIdx].enType) {
        u32VarSize = sizeof(HI_U8);
    } else {
        u32VarSize = sizeof(HI_U32);
    }

    /*fill src data*/
    if (SVP_BLOB_TYPE_SEQ_S32 ==
        pstNnieParam->astSegData[u32SegIdx].astSrc[u32NodeIdx].enType) {
        u32Dim = pstNnieParam->astSegData[u32SegIdx]
                     .astSrc[u32NodeIdx]
                     .unShape.stSeq.u32Dim;
        u32Stride =
            pstNnieParam->astSegData[u32SegIdx].astSrc[u32NodeIdx].u32Stride;
        pu32StepAddr = SAMPLE_SVP_NNIE_CONVERT_64BIT_ADDR(
            HI_U32, pstNnieParam->astSegData[u32SegIdx]
                        .astSrc[u32NodeIdx]
                        .unShape.stSeq.u64VirAddrStep);
        pu8PicAddr = SAMPLE_SVP_NNIE_CONVERT_64BIT_ADDR(
            HI_U8,
            pstNnieParam->astSegData[u32SegIdx].astSrc[u32NodeIdx].u64VirAddr);
        for (n = 0;
             n < pstNnieParam->astSegData[u32SegIdx].astSrc[u32NodeIdx].u32Num;
             n++) {
            for (i = 0; i < *(pu32StepAddr + n); i++) {
                s32Ret = fread(pu8PicAddr, u32Dim * u32VarSize, 1, fp);
                SAMPLE_SVP_CHECK_EXPR_GOTO(1 != s32Ret, FAIL,
                                           SAMPLE_SVP_ERR_LEVEL_ERROR,
                                           "Error,Read image file failed!\n");
                pu8PicAddr += u32Stride;
            }
            u32TotalStepNum += *(pu32StepAddr + n);
        }
        SAMPLE_COMM_SVP_FlushCache(
            pstNnieParam->astSegData[u32SegIdx].astSrc[u32NodeIdx].u64PhyAddr,
            SAMPLE_SVP_NNIE_CONVERT_64BIT_ADDR(
                HI_VOID, pstNnieParam->astSegData[u32SegIdx]
                             .astSrc[u32NodeIdx]
                             .u64VirAddr),
            u32TotalStepNum * u32Stride);
    } else {
        u32Height = pstNnieParam->astSegData[u32SegIdx]
                        .astSrc[u32NodeIdx]
                        .unShape.stWhc.u32Height;
        u32Width = pstNnieParam->astSegData[u32SegIdx]
                       .astSrc[u32NodeIdx]
                       .unShape.stWhc.u32Width;
        u32Chn = pstNnieParam->astSegData[u32SegIdx]
                     .astSrc[u32NodeIdx]
                     .unShape.stWhc.u32Chn;
        u32Stride =
            pstNnieParam->astSegData[u32SegIdx].astSrc[u32NodeIdx].u32Stride;
        pu8PicAddr = SAMPLE_SVP_NNIE_CONVERT_64BIT_ADDR(
            HI_U8,
            pstNnieParam->astSegData[u32SegIdx].astSrc[u32NodeIdx].u64VirAddr);
        if (SVP_BLOB_TYPE_YVU420SP ==
            pstNnieParam->astSegData[u32SegIdx].astSrc[u32NodeIdx].enType) {
            for (n = 0;
                 n <
                 pstNnieParam->astSegData[u32SegIdx].astSrc[u32NodeIdx].u32Num;
                 n++) {
                for (i = 0; i < u32Chn * u32Height / 2; i++) {
                    s32Ret = fread(pu8PicAddr, u32Width * u32VarSize, 1, fp);
                    SAMPLE_SVP_CHECK_EXPR_GOTO(
                        1 != s32Ret, FAIL, SAMPLE_SVP_ERR_LEVEL_ERROR,
                        "Error,Read image file failed!\n");
                    pu8PicAddr += u32Stride;
                }
            }
        } else if (SVP_BLOB_TYPE_YVU422SP == pstNnieParam->astSegData[u32SegIdx]
                                                 .astSrc[u32NodeIdx]
                                                 .enType) {
            for (n = 0;
                 n <
                 pstNnieParam->astSegData[u32SegIdx].astSrc[u32NodeIdx].u32Num;
                 n++) {
                for (i = 0; i < u32Height * 2; i++) {
                    s32Ret = fread(pu8PicAddr, u32Width * u32VarSize, 1, fp);
                    SAMPLE_SVP_CHECK_EXPR_GOTO(
                        1 != s32Ret, FAIL, SAMPLE_SVP_ERR_LEVEL_ERROR,
                        "Error,Read image file failed!\n");
                    pu8PicAddr += u32Stride;
                }
            }
        } else {
            for (n = 0;
                 n <
                 pstNnieParam->astSegData[u32SegIdx].astSrc[u32NodeIdx].u32Num;
                 n++) {
                for (i = 0; i < u32Chn; i++) {
                    for (j = 0; j < u32Height; j++) {
                        s32Ret =
                            fread(pu8PicAddr, u32Width * u32VarSize, 1, fp);
                        SAMPLE_SVP_CHECK_EXPR_GOTO(
                            1 != s32Ret, FAIL, SAMPLE_SVP_ERR_LEVEL_ERROR,
                            "Error,Read image file failed!\n");
                        pu8PicAddr += u32Stride;
                    }
                }
            }
        }
        SAMPLE_COMM_SVP_FlushCache(
            pstNnieParam->astSegData[u32SegIdx].astSrc[u32NodeIdx].u64PhyAddr,
            SAMPLE_SVP_NNIE_CONVERT_64BIT_ADDR(
                HI_VOID, pstNnieParam->astSegData[u32SegIdx]
                             .astSrc[u32NodeIdx]
                             .u64VirAddr),
            pstNnieParam->astSegData[u32SegIdx].astSrc[u32NodeIdx].u32Num *
                u32Chn * u32Height * u32Stride);
    }

    fclose(fp);
    return HI_SUCCESS;
FAIL:

    fclose(fp);
    return HI_FAILURE;
}

static HI_S32 SAMPLE_SVP_NNIE_Forward(
    SAMPLE_SVP_NNIE_PARAM_S *pstNnieParam,
    SAMPLE_SVP_NNIE_INPUT_DATA_INDEX_S *pstInputDataIdx,
    SAMPLE_SVP_NNIE_PROCESS_SEG_INDEX_S *pstProcSegIdx, HI_BOOL bInstant) {
    HI_S32 s32Ret = HI_SUCCESS;
    HI_U32 i = 0, j = 0;
    HI_BOOL bFinish = HI_FALSE;
    SVP_NNIE_HANDLE hSvpNnieHandle = 0;
    HI_U32 u32TotalStepNum = 0;
    SAMPLE_SVP_NIE_PERF_STAT_DEF_VAR()

    SAMPLE_COMM_SVP_FlushCache(
        pstNnieParam->astForwardCtrl[pstProcSegIdx->u32SegIdx]
            .stTskBuf.u64PhyAddr,
        SAMPLE_SVP_NNIE_CONVERT_64BIT_ADDR(
            HI_VOID, pstNnieParam->astForwardCtrl[pstProcSegIdx->u32SegIdx]
                         .stTskBuf.u64VirAddr),
        pstNnieParam->astForwardCtrl[pstProcSegIdx->u32SegIdx]
            .stTskBuf.u32Size);

    SAMPLE_SVP_NNIE_PERF_STAT_BEGIN()
    for (i = 0;
         i < pstNnieParam->astForwardCtrl[pstProcSegIdx->u32SegIdx].u32DstNum;
         i++) {
        if (SVP_BLOB_TYPE_SEQ_S32 ==
            pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                .astDst[i]
                .enType) {
            for (j = 0; j < pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                                .astDst[i]
                                .u32Num;
                 j++) {
                u32TotalStepNum +=
                    *(SAMPLE_SVP_NNIE_CONVERT_64BIT_ADDR(
                          HI_U32,
                          pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                              .astDst[i]
                              .unShape.stSeq.u64VirAddrStep) +
                      j);
            }
            SAMPLE_COMM_SVP_FlushCache(
                pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                    .astDst[i]
                    .u64PhyAddr,
                SAMPLE_SVP_NNIE_CONVERT_64BIT_ADDR(
                    HI_VOID, pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                                 .astDst[i]
                                 .u64VirAddr),
                u32TotalStepNum *
                    pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                        .astDst[i]
                        .u32Stride);

        } else {
            SAMPLE_COMM_SVP_FlushCache(
                pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                    .astDst[i]
                    .u64PhyAddr,
                SAMPLE_SVP_NNIE_CONVERT_64BIT_ADDR(
                    HI_VOID, pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                                 .astDst[i]
                                 .u64VirAddr),
                pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                        .astDst[i]
                        .u32Num *
                    pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                        .astDst[i]
                        .unShape.stWhc.u32Chn *
                    pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                        .astDst[i]
                        .unShape.stWhc.u32Height *
                    pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                        .astDst[i]
                        .u32Stride);
        }
    }

    /*set input blob according to node name*/
    if (pstInputDataIdx->u32SegIdx != pstProcSegIdx->u32SegIdx) {
        for (i = 0;
             i <
             pstNnieParam->pstModel->astSeg[pstProcSegIdx->u32SegIdx].u16SrcNum;
             i++) {
            for (j = 0;
                 j < pstNnieParam->pstModel->astSeg[pstInputDataIdx->u32SegIdx]
                         .u16DstNum;
                 j++) {
                if (0 == strncmp(pstNnieParam->pstModel
                                     ->astSeg[pstInputDataIdx->u32SegIdx]
                                     .astDstNode[j]
                                     .szName,
                                 pstNnieParam->pstModel
                                     ->astSeg[pstProcSegIdx->u32SegIdx]
                                     .astSrcNode[i]
                                     .szName,
                                 SVP_NNIE_NODE_NAME_LEN)) {
                    pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                        .astSrc[i] =
                        pstNnieParam->astSegData[pstInputDataIdx->u32SegIdx]
                            .astDst[j];
                    break;
                }
            }
            SAMPLE_SVP_CHECK_EXPR_RET(
                (j == pstNnieParam->pstModel->astSeg[pstInputDataIdx->u32SegIdx]
                          .u16DstNum),
                HI_FAILURE, SAMPLE_SVP_ERR_LEVEL_ERROR,
                "Error,can't find %d-th seg's %d-th src blob!\n",
                pstProcSegIdx->u32SegIdx, i);
        }
    }

    /*NNIE_Forward*/
    SAMPLE_SVP_NNIE_PERF_STAT_BEGIN()
    s32Ret = HI_MPI_SVP_NNIE_Forward(
        &hSvpNnieHandle,
        pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx].astSrc,
        pstNnieParam->pstModel,
        pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx].astDst,
        &pstNnieParam->astForwardCtrl[pstProcSegIdx->u32SegIdx], bInstant);
    SAMPLE_SVP_CHECK_EXPR_RET(HI_SUCCESS != s32Ret, s32Ret,
                              SAMPLE_SVP_ERR_LEVEL_ERROR,
                              "Error,HI_MPI_SVP_NNIE_Forward failed!\n");

    if (bInstant) {
        /*Wait NNIE finish*/
        while (HI_ERR_SVP_NNIE_QUERY_TIMEOUT ==
               (s32Ret = HI_MPI_SVP_NNIE_Query(
                    pstNnieParam->astForwardCtrl[pstProcSegIdx->u32SegIdx]
                        .enNnieId,
                    hSvpNnieHandle, &bFinish, HI_TRUE))) {
            usleep(100);
            SAMPLE_SVP_TRACE(SAMPLE_SVP_ERR_LEVEL_INFO,
                             "HI_MPI_SVP_NNIE_Query Query timeout!\n");
        }
    }
    u32TotalStepNum = 0;

    SAMPLE_SVP_NNIE_PERF_STAT_BEGIN()
    for (i = 0;
         i < pstNnieParam->astForwardCtrl[pstProcSegIdx->u32SegIdx].u32DstNum;
         i++) {
        if (SVP_BLOB_TYPE_SEQ_S32 ==
            pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                .astDst[i]
                .enType) {
            for (j = 0; j < pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                                .astDst[i]
                                .u32Num;
                 j++) {
                u32TotalStepNum +=
                    *(SAMPLE_SVP_NNIE_CONVERT_64BIT_ADDR(
                          HI_U32,
                          pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                              .astDst[i]
                              .unShape.stSeq.u64VirAddrStep) +
                      j);
            }
            SAMPLE_COMM_SVP_FlushCache(
                pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                    .astDst[i]
                    .u64PhyAddr,
                SAMPLE_SVP_NNIE_CONVERT_64BIT_ADDR(
                    HI_VOID, pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                                 .astDst[i]
                                 .u64VirAddr),
                u32TotalStepNum *
                    pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                        .astDst[i]
                        .u32Stride);

        } else {
            SAMPLE_COMM_SVP_FlushCache(
                pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                    .astDst[i]
                    .u64PhyAddr,
                SAMPLE_SVP_NNIE_CONVERT_64BIT_ADDR(
                    HI_VOID, pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                                 .astDst[i]
                                 .u64VirAddr),
                pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                        .astDst[i]
                        .u32Num *
                    pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                        .astDst[i]
                        .unShape.stWhc.u32Chn *
                    pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                        .astDst[i]
                        .unShape.stWhc.u32Height *
                    pstNnieParam->astSegData[pstProcSegIdx->u32SegIdx]
                        .astDst[i]
                        .u32Stride);
        }
    }

    return s32Ret;
}

bool eb3516Yolov3Init(const char *model_file, const char *pic_file) {
    HI_S32 ret = HI_SUCCESS;

    SAMPLE_SVP_NNIE_CFG_S stNnieCfg = {0};
    SAMPLE_SVP_NNIE_INPUT_DATA_INDEX_S stInputDataIdx = {0};
    SAMPLE_SVP_NNIE_PROCESS_SEG_INDEX_S stProcSegIdx = {0};
    HI_FLOAT f32PrintResultThresh = 0.0f;

    /*Set configuration parameter*/
    f32PrintResultThresh = 0.8f;
    stNnieCfg.pszPic= pic_file;
    stNnieCfg.u32MaxInputNum = 1; //max input image num in each batch
    stNnieCfg.u32MaxRoiNum = 0;
    stNnieCfg.aenNnieCoreId[0] = SVP_NNIE_ID_0;//set NNIE core

    // Get logger
    log_nnie = zlog_get_category("eb3516_nnie");
    if (!log_nnie) {
        return false;
    }

    // Init Sys
    SAMPLE_COMM_SVP_CheckSysInit();
    zlog_info(log_nnie, "Sys succeed to init");

    // Load model
    ret = SAMPLE_COMM_SVP_NNIE_LoadModel((HI_CHAR *)model_file, &g_stNnieModel);
    if (HI_SUCCESS != ret) {
        zlog_error(log_nnie, "NNIE failed to load model");
        goto EXIT_SYS;
    }
    zlog_info(log_nnie, "Model succeed to be loaded");

    // Init parameter
    g_stNnieParam.pstModel = &g_stNnieModel.stModel;
    ret = SAMPLE_SVP_NNIE_Yolov3_ParamInit(&stNnieCfg, &g_stNnieParam,
                                                &g_stYolov3Param);
    if (HI_SUCCESS != ret) {
        zlog_error(log_nnie, "NNIE failed to init parameter");
        goto EXIT_DEINIT;
    }

    // Read src pic
    stInputDataIdx.u32SegIdx = 0;
    stInputDataIdx.u32NodeIdx = 0;
    ret = SAMPLE_SVP_NNIE_FillSrcData(&stNnieCfg, &g_stNnieParam,
                                           &stInputDataIdx);
    if (HI_SUCCESS != ret) {
        zlog_error(log_nnie, "NNIE failed to read picture");
        goto EXIT_DEINIT;
    }

    // NNIE Forward
    stProcSegIdx.u32SegIdx = 0;
    ret = SAMPLE_SVP_NNIE_Forward(&g_stNnieParam, &stInputDataIdx,
                                       &stProcSegIdx, HI_TRUE);
    if (HI_SUCCESS != ret) {
        zlog_error(log_nnie, "NNIE failed to start forward");
        goto EXIT_DEINIT;
    }

    // Get Result
    ret = SAMPLE_SVP_NNIE_Yolov3_GetResult(&g_stNnieParam, &g_stYolov3Param);
    if (HI_SUCCESS != ret) {
        zlog_error(log_nnie, "NNIE failed to get result");
        goto EXIT_DEINIT;
    }

    // Print Result
    (void)SAMPLE_SVP_NNIE_Detection_PrintResult(
        &g_stYolov3Param.stDstScore, &g_stYolov3Param.stDstRoi,
        &g_stYolov3Param.stClassRoiNum, f32PrintResultThresh);

EXIT_DEINIT:
    SAMPLE_SVP_NNIE_Yolov3_Deinit(&g_stNnieParam, &g_stYolov3Param,
                                  &g_stNnieModel);
EXIT_SYS:
    SAMPLE_COMM_SVP_CheckSysExit();
    return false;
}

void eb3516Yolov3Exit(void) {
    SAMPLE_SVP_NNIE_Yolov3_Deinit(&g_stNnieParam, &g_stYolov3Param,
                                  &g_stNnieModel);
    memset(&g_stNnieParam, 0, sizeof(SAMPLE_SVP_NNIE_PARAM_S));
    memset(&g_stYolov3Param, 0,
           sizeof(SAMPLE_SVP_NNIE_YOLOV3_SOFTWARE_PARAM_S));
    memset(&g_stNnieModel, 0, sizeof(SAMPLE_SVP_NNIE_MODEL_S));
    SAMPLE_COMM_SVP_CheckSysExit();

    zlog_info(log_nnie, "Yolov3 Exit Successfully");
}
